/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.message.api;

import nl.cloudfarming.client.message.node.MessageNodeFactory;
import java.util.List;
import javax.persistence.Query;
import nl.cloudfarming.client.model.CommonModelService;
import nl.cloudfarming.client.model.DataProviderImpl;
import org.openide.util.lookup.ServiceProvider;
import org.openide.util.lookup.ServiceProviders;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import javax.swing.ActionMap;
import org.openide.explorer.ExplorerManager;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;

/**
 * Service for messaging module
 * 
 * @author Gerben Feenstra & Merijn Zengers
 */
@ServiceProviders(value = {
    @ServiceProvider(service = DataProviderImpl.class),
    @ServiceProvider(service = MessageService.class)
})
public class MessageService extends CommonModelService {

    public static final String PROPERTY_ADDED = "added";
    private final List<Message> volatileMessages = new ArrayList<Message>();
    private final PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
    private final InstanceContent content = new InstanceContent();
    private final Lookup lookup = new AbstractLookup(content);

    public MessageService() {
        //volatileMessages = Collections.synchronizedList(findAllMessages());
    }

    /**
     * Find and return all messages in the database
     * @return List of messages
     */
    public List<Message> getPersistedMessages() {
        startTransaction();
        Query query = getEntityManager().createNamedQuery(Message.Q_FIND_ALL_MESSAGES);
        return query.getResultList();
    }

    public List<Message> getVolatileMessages() {
        return volatileMessages;
    }

    public synchronized List<Message> getAllMessages() {
        List<Message> persistedMessages = getPersistedMessages();
        List<Message> allMessages = new ArrayList<Message>();
        allMessages.addAll(persistedMessages);
        allMessages.addAll(volatileMessages);
        return allMessages;
    }

    /**
     * Returns the module name
     * @return Name of this module
     */
    @Override
    public String getModuleName() {
        return MessageApiModule.MODULE_NAME;
    }

    /**
     * Adds a message to the inbox.
     * @param message the message to be add
     */
    public void addMessageToMessageList(Message message) {
        if (!volatileMessages.contains(message)) {
            volatileMessages.add(message);
            propertyChangeSupport.firePropertyChange(PROPERTY_ADDED, null, message);
        }
    }

    /**
     * Create or Update existing message. For example set a message as "read"
     * @param message The message instance to merge with the existing persisted message
     */
    void createOrUpdateMessage(Message message) {
        if (message.getId() == 0) {
            mergeAndPersistInTransaction(message);
        } else {
            startTransaction();
            getEntityManager().merge(message);
            commit();
        }
    }

    /**
     * Not used for this module
     */
    @Override
    protected void initExplorer() {
        Node rootNode = new AbstractNode(Children.create(new MessageNodeFactory(), true));
        ExplorerManager manager = new ExplorerManager();
        manager.setRootContext(rootNode);
        manager.getRootContext().setDisplayName("messages");

        setExplorerManager(manager, new ActionMap());
    }

    /**
     * Adds property change listener which listens to messages added to this inbox
     * @param pcl the property change listener
     */
    public void addMessagePropertyChangeListener(PropertyChangeListener pcl) {
        propertyChangeSupport.addPropertyChangeListener(PROPERTY_ADDED, pcl);
    }

    public void removeMessagePropertyChangeListener(PropertyChangeListener pcl) {
        propertyChangeSupport.removePropertyChangeListener(PROPERTY_ADDED, pcl);
    }

    @Override
    public Lookup getLookup() {
        return this.lookup;
    }
}
